/*******************************************************************************
 * (c) 2007 - http://www.hofrichter.net - sven hofrichter - pspad@hofrichter.net
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * author ......... : Sven Hofrichter, Vincent Caron
 * date ........... : 2007-10-12
 * version ........ : 1.3
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * description .... : this (user-)script reformats a XML-structure into a for
 *                    human readable style. It is very useful to bring one-
 *                    line-XML-files into a structured format.
 * introduction ... : open the xml-file, that you want to change, in PSPad and
 *                    (optional) use the following short-cut:
 *                    ~ <CTRL><SHIFT><ALT><X> ... formated xml-structure will
 *                                                replace the original one in
 *                                                current editortab
 *                    ~ <CTRL><SHIFT><ALT><H> ... like before, additionaly the
 *                                                encoding will happen
 *                    ~ <CTRL><SHIFT><ALT><G> ... like before, but the decoding
 *                                                will happen
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * history:
 * 2006-09-18 - 0.1 - hofsve - initial version
 * 2006-10-02 - 0.2 - hofsve - no empty lines at the beginning
 * 2006-10-06 - 0.3 - hofsve - "wellformed" xml-structures won't be changed
 * 2007-02-02 - 0.4 - hofsve - it's possible to choose the number of spaces for
 *                             the indent
 * 2007-08-07 - 0.5 - hofsve - the script formats the selected text only or the
 *                             whole text, if nothing was selected
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * 2007-09-20 - 1.0 - carvin
 *                  - hofsve - full redesign for a better performance and compa-
 *                             tibility to (X)HTML. Unclosed link- or meta-tags
 *                             don't desturb the structuring. Support of self-
 *                             closed tags (e.g. "<br />") is implemented.
 *                             The indenting with tab's is implemented too.
 * 2007-09-26 - 1.1 - hofsve - fixed: a bug while formating xml-structures,
 *                             those don't contain a line-break didn't format
 *                             the xml-structure.
 *                             fixed: The script didn't handle closing tags
 *                             correctly, if there were more than two without
 *                             an opening tag between.
 * 2007-10-03 - 1.2 - carvin - fixed: a bug occured in version 1.1 while
 *                             formating "<?xml ...".
 * 2007-10-12 - 1.3 - hofsve - Character-encoding/-decoding for (x)html-files
 *                             included. The reformater is enabled for this
 *                             function too.
 ******************************************************************************/

var module_name = "XMLFormat";
var module_ver = "1.3";
function xmlFormat() {
	var m_editor = newEditor();
	m_editor.assignActiveEditor();
	if( m_editor.selText().length>0 ) {
		m_editor.selText( FormatXML( m_editor.selText(), 0 ) );
	}
	else {
		m_editor.Text( FormatXML( m_editor.Text(), 0 ) );
	}
}
function htmlEncode() {
	var m_editor = newEditor();
	m_editor.assignActiveEditor();
	if( m_editor.selText().length>0 ) {
		m_editor.selText( FormatXML( m_editor.selText(), 1 ) );
	}
	else {
		m_editor.Text( FormatXML( m_editor.Text(), 1 ) );
	}
}
function htmlDecode() {
	var m_editor = newEditor();
	m_editor.assignActiveEditor();
	if( m_editor.selText().length>0 ) {
		m_editor.selText( FormatXML( m_editor.selText(), 2 ) );
	}
	else {
		m_editor.Text( FormatXML( m_editor.Text(), 2 ) );
	}
}
function FormatXML( m_text, m_flag ) {
  if( m_flag==2 ) {
    m_text = m_text.replace( /\&\#0*(\d+);/g, decoderHelper );
  }
	var m_default_depth = 0; // default is "tab"
	var m_chosen_depth  = null;
	try {
		// if the version of PSPad is not a newer one, the following will fail:
		m_chosen_depth = InputText( "Please enter the depth (0=Tab):", m_default_depth );
	}
	catch(e) {
		// setting up the default, if there was something happen before
		m_chosen_depth = m_default_depth;
	}
	m_indent_sign = " ";
	if( m_chosen_depth == 0 ) {
		m_chosen_depth = 1;
		m_indent_sign  = "\t";
	}
	var m_spaces = m_indent_sign;
	for( var m_curr=0; m_curr<100; m_curr++ ) {
		m_spaces += m_indent_sign;
	}
	var mRegExp = /([\n\r])/;
	mRegExp.exec(m_text);
	m_new_line = RegExp.$1;
	if( m_new_line==null || m_new_line=="" ) {
    m_new_line = '\r\n';
    var mMsg = "[ATTENTION]\nNo newline-marker found! This happens only on one-line-selections.";
        mMsg+= "\nThe formater will use the DOS-sequence for a new line '\\r\\n' (CR+LF).";
        mMsg+= "\n\nPlease use the 'Format'-menu of PSPad to handle/correct the format for the"
        mMsg+= "\nwhole file after the formater ran.";
        mMsg+= "\nYou can see the original format (DOS, UNIX or MAC) in the status-line of PSPad.";
    echo( mMsg );
  }
	// please do not change brackets in strings or escaped ones:
  m_text = m_text.replace(/^\s*<\?xml/gi, "####xml_version_tag####" );
  m_text = m_text.replace(/(<\/[^>]+>)\s*(<\/[^>]+>)/g, "$1$2" );
  m_text = m_text.replace(/(<\/[^>]+>)([^<]+)(<\/[^>]+>)/g, "$1"+m_new_line+"$2"+m_new_line+"$3" );
// 	m_text = m_text.replace(/(<\/[^>]+>)(<\/[^>]+>)/g, "$1"+m_new_line+"$2" );
// 	m_text = m_text.replace(/(<\/[^>]+>)(<\/[^>]+>)/g, "$1"+m_new_line+"$2" );
	m_text = m_text.replace(/(<\/[^>]+>)/g, "$1"+m_new_line );
	m_text = m_text.replace(/<\//g, "####close_tag####"  );
	m_text = m_text.replace(/<\s*(link|meta)(\s)/gi, "####unclosed_tag####$1$2"  );
	m_text = m_text.replace(/<\!/g, "####ignore_tag####"  );

	// cleanup the bracket-structure and prepare it for a clean splitting
	m_text = m_text.replace(/\s*</g,m_new_line+"<");
	m_text = m_text.replace(/([\r\n]+)/, m_new_line );

	m_br_array = m_text.split( /[\r\n]/ );
	var m_return = "";
	var m_current_depth = 0;
	// every line and every bracket is now in it's own line
	for( var i=0; i<m_br_array.length; i++ ) {
		// trim the line
		m_br_array[i] = m_br_array[i].replace(/^\s*/g,"").replace(/\s*$/g,"");
		// ignore empty lines
		if( m_br_array[i].replace(/\s+/g,"").length==0 ) {
			continue;
		}
		if( i>0 ) {
			m_return += m_new_line;
		}
    var m_op = 0;
    var m_cl = 0;
		var m_matched = m_br_array[i].match( /(\/\>|\#\#\#\#close_tag\#\#\#\#)/g );
    if( m_matched ) {
      m_cl = m_matched.length;
    }
		m_matched = m_br_array[i].match( /</g );
    if( m_matched ) {
      m_op = m_matched.length;
    }

		if( m_br_array[i].indexOf("####close_tag####")==0 && m_cl>0 && m_op==0 ) {
			m_current_depth = m_current_depth - 1;
			m_cl = m_cl - 1;
		}

		if( m_chosen_depth>0 && m_current_depth>0 ) {
			m_return += m_spaces.substr( 0, m_current_depth*m_chosen_depth );
		}
		m_return += m_br_array[i];
    m_current_depth = m_current_depth + m_op - m_cl;
	}
	m_return = m_return.replace( /####close_tag####/g, "</" );
	m_return = m_return.replace( /####unclosed_tag####(link|meta)/gi, "<$1" );
	m_return = m_return.replace( /####ignore_tag####/g, "<!" );
	m_return = m_return.replace( /####xml_version_tag####/g, "<?" );
  if( m_flag==1 ) {
    m_return = encoderHelper( m_return );
  }
	return m_return;
}
function encoderHelper( m_line ) {
  var m_is_tag   = false;
  var m_array    = m_line.split( /<\/*body/ ); //*/
  if( m_array.length!=3 ) {
    m_array = new Array( "", m_line, "" );
  } else {
      m_array[0]+= "<body" + m_array[1].substr(0,m_array[1].indexOf(">"));
      m_array[1] = m_array[1].substr(m_array[1].indexOf(">"),m_array[1].length);
      m_array[2] = "</body" + m_array[2];
  }
  m_array[1] = m_array[1].replace( /\&([\w\d]+);/g, "123amp456$1;" );
  m_array[1] = m_array[1].replace( /\&\#([\w\d]+);/g, "456amp789$1;" );
  var m_return   = "";
  for( var j=0; j<m_array[1].length; j++ ) {
    var mSign = m_array[1].charAt(j);
    var mCode = m_array[1].charCodeAt(j);
    if( mSign=="<" ) {
      m_return += m_array[1].charAt(j);
      m_is_tag = true;
    } else if( mSign==">" ) {
      m_return += mSign;
      m_is_tag = false;
    } else if( m_is_tag==true ) {
      m_return += mSign;
    } else if( ( mCode>=33 && mCode<=47 ) || ( mCode>=91 && mCode<=96 ) || mCode>=123 ) {
      if( mCode<10 ) {
        m_return += "&#00" + mCode + ";";
      }
      else if( mCode<100 ) {
        m_return += "&#0" + mCode + ";";
      }
      else {
        m_return += "&#" + mCode + ";";
      }
    } else {
      m_return += mSign;
    }
  }
  m_return = m_return.replace(/123amp456/g,"&");
  m_return = m_return.replace(/456amp789/g,"&#");
  return m_array[0] + m_return + m_array[2];
}
function decoderHelper( str, dollar1, pos, src ) {
   return String.fromCharCode(dollar1);
}
function Init() {
  addMenuItem("-", "Format Code", "");
	addMenuItem( "Format tags", "(X)HTML-/XML-Format (v"+module_ver+")", "xmlFormat", "CTRL+ALT+SHIFT+X");
	addMenuItem( "Format tags and encode special chars", "(X)HTML-/XML-Format (v"+module_ver+")", "htmlEncode", "CTRL+ALT+SHIFT+H");
	addMenuItem( "Format tags and decode special chars", "(X)HTML-/XML-Format (v"+module_ver+")", "htmlDecode", "CTRL+ALT+SHIFT+G");
  addMenuItem("-", "Format Code", "");
}
